<?php
/*
Plugin Name: SEO Image Link
Plugin URI:  https://dirschl.com/
Description: Zeigt per Shortcode [seo-img-link site="webseite.com" style="image" bzw. "screenshot" odder "" für beides durch Mouseover title="Optionale Überschrift"] die Metadaten (Title, Description, Bild) der angegebenen Webseite an.
Version:     1.0
Author:      Johann Dirschl
Author URI:  https://dirschl.com/
License:     GPL2
*/

// Verhindere direkten Aufruf.
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Holt per wp_remote_get den HTML-Code einer URL.
 *
 * @param string $url URL der abzurufenden Webseite.
 * @return string|WP_Error HTML-Inhalt oder WP_Error bei Fehler.
 */
function seoi_get_remote_html( $url ) {
    $response = wp_remote_get( $url, array(
        'timeout' => 10,
    ) );

    if ( is_wp_error( $response ) ) {
        return $response;
    }

    $code = wp_remote_retrieve_response_code( $response );
    if ( 200 !== $code ) {
        return new WP_Error( 'invalid_response', __( 'Ungültige Antwort vom Server', 'seo-img-link' ) );
    }

    $body = wp_remote_retrieve_body( $response );
    return $body;
}

/**
 * Parst den HTML-Code und extrahiert die Metadaten (og:title, og:description, og:image).
 *
 * @param string $html HTML-Inhalt.
 * @return array Array mit den Keys 'title', 'description' und 'image'.
 */
function seoi_parse_meta_data( $html ) {
    $meta = array(
        'title'       => '',
        'description' => '',
        'image'       => '',
    );

    if ( empty( $html ) ) {
        return $meta;
    }

    libxml_use_internal_errors( true );
    $doc = new DOMDocument();
    $doc->loadHTML( $html );
    libxml_clear_errors();

    $metas = $doc->getElementsByTagName( 'meta' );
    foreach ( $metas as $tag ) {
        $property = strtolower( $tag->getAttribute( 'property' ) );
        $name     = strtolower( $tag->getAttribute( 'name' ) );
        $content  = $tag->getAttribute( 'content' );

        if ( 'og:title' === $property && empty( $meta['title'] ) ) {
            $meta['title'] = esc_html( $content );
        }
        if ( 'og:description' === $property && empty( $meta['description'] ) ) {
            $meta['description'] = esc_html( $content );
        }
        if ( 'og:image' === $property && empty( $meta['image'] ) ) {
            $meta['image'] = esc_url_raw( $content );
        }
        // Fallback: Sollte der <meta name="title"> Tag existieren.
        if ( empty( $meta['title'] ) && 'title' === $name ) {
            $meta['title'] = esc_html( $content );
        }
    }

    // Zusätzlicher Fallback über den <title>-Tag.
    if ( empty( $meta['title'] ) ) {
        $nodes = $doc->getElementsByTagName( 'title' );
        if ( $nodes->length > 0 ) {
            $meta['title'] = esc_html( $nodes->item( 0 )->nodeValue );
        }
    }

    return $meta;
}

/**
 * Shortcode-Funktion: [seo-img-link site="webseite.com" style="image" bzw. "screenshot" title="Überschrift"]
 *
 * @param array $atts Shortcode-Attribute.
 * @return string HTML-Ausgabe.
 */
function seoi_shortcode( $atts ) {
    // Parameter: site (Pflicht), style (optional, Standard: image) und title (optional für h2)
    $atts = shortcode_atts( array(
        'site'  => '',
        'style' => 'image',
        'title' => '',
    ), $atts, 'seo-img-link' );

    $site         = esc_url_raw( trim( $atts['site'] ) );
    $style        = strtolower( trim( $atts['style'] ) );
    $custom_title = trim( $atts['title'] );

    // Falls keine Seite angegeben wurde, Rückgabe eines Fehlermeldungstexts.
    if ( empty( $site ) ) {
        return '<p>' . esc_html__( 'Es wurde keine Webseite angegeben.', 'seo-img-link' ) . '</p>';
    }

    // Falls kein Protokoll vorhanden ist, wird http:// vorangestellt.
    if ( false === strpos( $site, 'http://' ) && false === strpos( $site, 'https://' ) ) {
        $site = 'http://' . $site;
    }

    // Standardmäßig, falls style leer ist, auf "image" setzen.
    if ( empty( $style ) ) {
        $style = 'image';
    }

    // Hole den HTML-Code der Seite.
    $html = seoi_get_remote_html( $site );
    if ( is_wp_error( $html ) ) {
        return '<p>' . esc_html__( 'Die Webseite konnte nicht abgerufen werden.', 'seo-img-link' ) . '</p>';
    }

    // Hole die Metadaten.
    $meta = seoi_parse_meta_data( $html );
    // Fallback, falls keine Daten gefunden wurden.
    $page_title       = ! empty( $meta['title'] ) ? $meta['title'] : esc_html__( 'Kein Titel gefunden', 'seo-img-link' );
    $page_description = ! empty( $meta['description'] ) ? $meta['description'] : esc_html__( 'Keine Beschreibung gefunden', 'seo-img-link' );

    // Definiere den Screenshot-URL mittels mShots.
    // Hier wird eine Breite von 600px angefragt – das eigentliche Layout erfolgt via CSS (width: 100%).
    $width          = 600;
    $screenshot_url = 'https://s.wordpress.com/mshots/v1/' . urlencode( $site ) . '?w=' . intval( $width );

    // Bestimme die primäre Bild-URL anhand des style-Parameters.
    if ( 'screenshot' === $style ) {
        $primary_image = $screenshot_url;
    } else {
        // style "image" (Standard) – nutze das Meta-Bild; falls nicht vorhanden, den Screenshot
        $primary_image = ! empty( $meta['image'] ) ? $meta['image'] : $screenshot_url;
    }

    // Starte den Output-Puffer.
    ob_start();
    ?>
    <div class="sil-wrapper">
        <?php if ( ! empty( $custom_title ) ) : ?>
            <h2 class="sil-heading"><?php echo esc_html( $custom_title ); ?></h2>
        <?php endif; ?>

        <div class="sil-image-wrapper">
            <?php if ( 'image' === $style ) : ?>
                <a href="<?php echo esc_url( $site ); ?>" title="<?php echo esc_attr( $page_title ); ?>">
                    <div class="sil-img-container">
                        <img src="<?php echo esc_url( $primary_image ); ?>" class="sil-image main-image" alt="<?php echo esc_attr( $page_title ); ?>" title="<?php echo esc_attr( $page_title ); ?>">
                        <img src="<?php echo esc_url( $screenshot_url ); ?>" class="sil-image screenshot-image" alt="<?php echo esc_attr( $page_title ); ?>" title="<?php echo esc_attr( $page_title ); ?>">
                    </div>
                </a>
            <?php else : ?>
                <!-- style "screenshot": Zeige ausschließlich den Screenshot -->
                <a href="<?php echo esc_url( $site ); ?>" title="<?php echo esc_attr( $page_title ); ?>">
                    <img src="<?php echo esc_url( $screenshot_url ); ?>" class="sil-image" style="width:100%; object-fit:cover; object-position:top;" alt="<?php echo esc_attr( $page_title ); ?>" title="<?php echo esc_attr( $page_title ); ?>">
                </a>
            <?php endif; ?>
        </div><!-- .sil-image-wrapper -->

        <h3 class="sil-page-title"><?php echo $page_title; ?></h3>
        <p class="sil-page-description"><?php echo $page_description; ?></p>
        <p class="sil-page-url">
            <a href="<?php echo esc_url( $site ); ?>" title="<?php echo esc_attr( $page_title ); ?>"><?php echo esc_html( $site ); ?></a>
        </p>
    </div><!-- .sil-wrapper -->
    <?php
    return ob_get_clean();
}
add_shortcode( 'seo-img-link', 'seoi_shortcode' );

/**
 * Lädt die externe CSS-Datei für das SEO Image Link Plugin.
 */
function seoi_enqueue_assets() {
    wp_enqueue_style(
        'sil-style',
        plugin_dir_url( __FILE__ ) . 'assets/css/seo-img-link.css',
        array(),
        '1.0'
    );
}
add_action( 'wp_enqueue_scripts', 'seoi_enqueue_assets' );
